(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 11.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     19911,        558]
NotebookOptionsPosition[     17488,        512]
NotebookOutlinePosition[     17949,        530]
CellTagsIndexPosition[     17906,        527]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell[".NET link sample of ICU64 for VICE v0.1.3", "Section",ExpressionUUID->"1bcb10e7-e31c-4f8b-b039-b6bad58ce3df"],

Cell["\<\
How to use:
- In the initialization cell bellow, edit the full paths to the files \
\[OpenCurlyDoubleQuote]ICU64.EXE\[CloseCurlyDoubleQuote] and \
\[OpenCurlyDoubleQuote]icu64.ini\[CloseCurlyDoubleQuote]
- Select Menu > Evaluation > Evaluate Initialization Cells, and wait ICU64 \
and emulator to start
- Evaluate one-by-one  the white cells only (while keep an eye in the screen \
of the emulator)
- When you finish and close everything, check in the Windows Task Manager for \
\[OpenCurlyDoubleQuote]InstallableNET.exe\[CloseCurlyDoubleQuote] and kill it \
(sometimes remains there)

~Mathfigure 2020.03.11\
\>", "Text",
 Background->RGBColor[
  1, 1, 0.85],ExpressionUUID->"64afa7cd-0d64-4e2d-8025-c684ef0be20d"],

Cell[CellGroupData[{

Cell["Initialization", "Subsection",ExpressionUUID->"ebe8a79b-4883-49ce-8e7e-73a14c3ed379"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"path", "=", "\"\<C:\\\\C64\\\\ICU64.EXE\>\""}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
   "pathini", "=", "\"\<C:\\\\C64\\\\WinVICE-3.2-x86\\\\icu64.ini\>\""}], 
   ";"}], "\[IndentingNewLine]"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Needs", "[", "\"\<NETLink`\>\"", "]"}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"SetDirectory", "[", 
   RowBox[{"DirectoryName", "[", "pathini", "]"}], "]"}], ";"}], "\n", 
 RowBox[{"LoadNETAssembly", "[", "path", "]"}], "\n", 
 RowBox[{"c64", "=", 
  RowBox[{"NETNew", "[", "\"\<icu64.VM\>\"", "]"}]}], "\n", 
 RowBox[{"main", "=", 
  RowBox[{"NETNew", "[", 
   RowBox[{"\"\<icu64.MainForm\>\"", ",", "pathini"}], 
   "]"}]}], "\[IndentingNewLine]", 
 RowBox[{"DoNETModeless", "[", "main", "]"}]}], "Input",
 InitializationCell->
  True,ExpressionUUID->"f84757bd-866c-4276-ac84-d2d381af7710"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Basic Examples", "Subsection",ExpressionUUID->"f85da292-8f9e-4f72-b74c-f62ad49ec021"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"peek", " ", "a", " ", "single", " ", "byte"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"c64", "@", 
   RowBox[{"Peek", "[", "1024", "]"}]}]}]], "Input",
 CellLabel->"In[17]:=",ExpressionUUID->"3752278d-d24b-4f09-ab7e-68d60110bd49"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"poke", " ", "a", " ", "single", " ", "byte"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{"c64", "@", 
   RowBox[{"Poke", "[", 
    RowBox[{"1024", ",", "1"}], "]"}]}]}]], "Input",
 CellLabel->"In[19]:=",ExpressionUUID->"635f98b5-0924-43e3-838a-883b72c49bd6"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"poke", " ", "a", " ", "range", " ", "of", " ", "bytes"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"c64", "@", 
   RowBox[{"Poke", "[", 
    RowBox[{"1024", ",", 
     RowBox[{"{", 
      RowBox[{"8", ",", "5", ",", "12", ",", "12", ",", "15"}], "}"}]}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[20]:=",ExpressionUUID->"a20a3cf2-503f-4cf6-8396-e8e5f5de7a58"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"peek", " ", "a", " ", "range", " ", "of", " ", "bytes"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"c64", "@", 
   RowBox[{"Peek", "[", 
    RowBox[{"1024", ",", "40"}], "]"}]}]}]], "Input",
 CellLabel->"In[21]:=",ExpressionUUID->"b9b24fb6-4014-400b-88cd-8547ae6257b7"]
}, Open  ]],

Cell[CellGroupData[{

Cell["More Examples", "Subsection",ExpressionUUID->"06f42f47-a6b1-40d1-881b-2cc1fb856233"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"some", " ", "handy", " ", "definitions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"poke", "[", 
     RowBox[{"a_", ",", "v__"}], "]"}], ":=", 
    RowBox[{"c64", "@", 
     RowBox[{"Poke", "[", 
      RowBox[{"a", ",", "v"}], "]"}]}]}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"peek", "[", 
     RowBox[{"a_", ",", "n___"}], "]"}], ":=", 
    RowBox[{"c64", "@", 
     RowBox[{"Peek", "[", 
      RowBox[{"a", ",", "n"}], "]"}]}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"rom", "=", "16^^10000"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"mmio", "=", "16^^20000"}], ";"}]}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[9]:=",ExpressionUUID->"767f6b63-6b8e-4c2f-acaf-548ff3bd2886"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{"peek", " ", "&"}], " ", "poke", " ", "in", " ", "MMIO"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{"poke", "[", 
   RowBox[{
    RowBox[{"mmio", "+", "55296"}], ",", 
    RowBox[{
     RowBox[{"peek", "[", 
      RowBox[{"mmio", "+", "55296"}], "]"}], "+", "3"}]}], "]"}]}]], "Input",
 CellLabel->"In[22]:=",ExpressionUUID->"7f79e117-34bb-4de3-b4b2-e4c1dcf6b0a1"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"peek", " ", "from", " ", "ROM"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"peek", "[", 
    RowBox[{
     RowBox[{"rom", "+", "16^^d000", "+", 
      RowBox[{"1", "*", "8"}]}], ",", "8"}], "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"IntegerDigits", "[", 
     RowBox[{"%", ",", "2", ",", "8"}], "]"}], "//", "Image"}]}]}]], "Input",
 CellLabel->"In[23]:=",ExpressionUUID->"39721d41-892f-4183-b86b-300a965719f2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"copy", " ", "ROM", " ", "to", " ", "RAM"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"poke", "[", 
    RowBox[{"16^^a000", ",", 
     RowBox[{"peek", "[", 
      RowBox[{
       RowBox[{"rom", "+", "16^^a000"}], ",", "16^^2000"}], "]"}]}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{"poke", "[", 
    RowBox[{"16^^e000", ",", 
     RowBox[{"peek", "[", 
      RowBox[{
       RowBox[{"rom", "+", "16^^e000"}], ",", "16^^2000"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[25]:=",ExpressionUUID->"f680b9e3-40f2-44f3-b7ea-cf97a9bf28fd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"show", " ", "the", " ", "whole", " ", "RAM"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Partition", "[", 
     RowBox[{
      RowBox[{"peek", "[", 
       RowBox[{"0", ",", "65536"}], "]"}], ",", "256"}], "]"}], "/", "255."}],
    "//", "Image"}]}]], "Input",
 CellLabel->"In[27]:=",ExpressionUUID->"1e6e0221-998f-4e68-aae7-41b8d15806bd"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "show", " ", "the", " ", "default", " ", "text", " ", "screen", " ", "in",
      " ", "real"}], "-", "time"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"chars", "=", 
     RowBox[{"Image", "/@", 
      RowBox[{"Partition", "[", 
       RowBox[{
        RowBox[{"IntegerDigits", "[", 
         RowBox[{
          RowBox[{"peek", "[", 
           RowBox[{
            RowBox[{"rom", "+", "16^^d000"}], ",", "2048"}], "]"}], ",", "2", 
          ",", "8"}], "]"}], ",", "8"}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Dynamic", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Partition", "[", 
       RowBox[{
        RowBox[{"chars", "[", 
         RowBox[{"[", 
          RowBox[{"1", "+", 
           RowBox[{"peek", "[", 
            RowBox[{"16^^400", ",", "1000"}], "]"}]}], "]"}], "]"}], ",", 
        "40"}], "]"}], "//", "ImageAssemble"}], ",", 
     RowBox[{"UpdateInterval", "\[Rule]", "0.1"}]}], "]"}]}]}]], "Input",
 CellLabel->"In[28]:=",ExpressionUUID->"8e620a02-1013-4ac5-930a-91079d55a0f7"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"dynamically", " ", "rotate", " ", "the", " ", 
    RowBox[{"character", " ", "'"}], 
    RowBox[{"A", "'"}], " ", 
    RowBox[{"(", 
     RowBox[{"look", " ", "at", " ", 
      RowBox[{"any", " ", "'"}], 
      RowBox[{"A", "'"}], " ", "in", " ", "the", " ", "C64", " ", "window"}], 
     ")"}]}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"t", "=", "0"}], ";", 
    RowBox[{"c", "=", "1"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"Dynamic", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"t", "+=", "0.1"}], ";", "\[IndentingNewLine]", 
     RowBox[{"poke", "[", 
      RowBox[{
       RowBox[{"rom", "+", "16^^d000", "+", 
        RowBox[{"8", "c"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{
         RowBox[{
          RowBox[{"ImageRotate", "[", 
           RowBox[{
            RowBox[{"chars", "[", 
             RowBox[{"[", 
              RowBox[{"1", "+", "c"}], "]"}], "]"}], ",", "t", ",", "Full"}], 
           "]"}], "//", "ImageData"}], "//", 
         RowBox[{
          RowBox[{"FromDigits", "[", 
           RowBox[{"#", ",", "2"}], "]"}], "&"}]}], "//", "Flatten"}]}], 
      "]"}]}], "\[IndentingNewLine]", "]"}]}]}]], "Input",
 CellLabel->"In[30]:=",ExpressionUUID->"5c23c635-f650-4e09-920f-e5a9bd1bdd1d"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"more", " ", "handy", " ", "definitions"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"out", "[", 
     RowBox[{"a_", ",", "v__"}], "]"}], ":=", 
    RowBox[{"c64", "@", 
     RowBox[{"Poke", "[", 
      RowBox[{
       RowBox[{"16^^20000", "+", "a"}], ",", "v"}], "]"}]}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"in", "[", 
     RowBox[{"a_", ",", "n___"}], "]"}], ":=", 
    RowBox[{"c64", "@", 
     RowBox[{"Peek", "[", 
      RowBox[{
       RowBox[{"16^^20000", "+", "a"}], ",", "n"}], "]"}]}]}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"set", " ", "graphics", " ", "mode", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"bank", "=", 
        RowBox[{
         RowBox[{"0", ".."}], "3"}]}], ",", " ", 
       RowBox[{"page", "=", 
        RowBox[{
         RowBox[{"0", ".."}], "1"}]}], ",", " ", 
       RowBox[{"colorpage", "=", 
        RowBox[{
         RowBox[{"0", ".."}], "15"}]}]}], ")"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"setgraphics", "[", 
     RowBox[{"bank_", ",", "page_", ",", "colorpage_"}], "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"out", "[", 
        RowBox[{"53265", ",", 
         RowBox[{
          RowBox[{"in", "[", "53265", "]"}], "~", "BitOr", "~", "32"}]}], 
        "]"}], ";", 
       RowBox[{"(*", 
        RowBox[{"BMM", "=", "1"}], "*)"}], "\[IndentingNewLine]", 
       RowBox[{"out", "[", 
        RowBox[{"56578", ",", 
         RowBox[{
          RowBox[{"in", "[", "56578", "]"}], "~", "BitOr", "~", "3"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"out", "[", 
        RowBox[{"56576", ",", 
         RowBox[{
          RowBox[{"in", "[", "56576", "]"}], "~", "BitAnd", "~", "252", "~", 
          "BitOr", "~", 
          RowBox[{"(", 
           RowBox[{"3", "-", "bank"}], ")"}]}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"out", "[", 
        RowBox[{"53272", ",", 
         RowBox[{
          RowBox[{"in", "[", "53272", "]"}], "~", "BitAnd", "~", "7", "~", 
          "BitOr", "~", 
          RowBox[{"(", 
           RowBox[{"16", "*", "colorpage"}], ")"}], "~", "BitOr", "~", 
          RowBox[{"(", 
           RowBox[{"8", "*", "page"}], ")"}]}]}], "]"}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"16^^4000", "*", "bank"}], "+", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"16^^2000", "*", "page"}], ",", 
          RowBox[{"16^^400", "*", "colorpage"}]}], "}"}]}]}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{
     "convert", " ", "an", " ", "Image", " ", "to", " ", "a", " ", "c64", " ",
       "hi"}], "-", 
     RowBox[{"res", " ", "bitmap"}]}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"bitmap", "[", "g_", "]"}], ":=", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "}"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"a", "=", 
        RowBox[{
         RowBox[{
          RowBox[{"ColorQuantize", "[", 
           RowBox[{
            RowBox[{"ImageResize", "[", 
             RowBox[{"g", ",", 
              RowBox[{"{", 
               RowBox[{"320", ",", "200"}], "}"}]}], "]"}], ",", "2"}], "]"}],
           "//", "Binarize"}], "//", "ImageData"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"b", "=", 
        RowBox[{"Partition", "[", 
         RowBox[{
          RowBox[{"a", "//", "Flatten"}], ",", "8"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"c", "=", 
        RowBox[{"Map", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"FromDigits", "[", 
            RowBox[{"#", ",", "2"}], "]"}], "&"}], ",", "b"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"d", "=", 
        RowBox[{"Partition", "[", 
         RowBox[{
          RowBox[{"Partition", "[", 
           RowBox[{"c", ",", "40"}], "]"}], ",", "8"}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Transpose", "[", 
         RowBox[{"d", ",", 
          RowBox[{"{", 
           RowBox[{"1", ",", "3", ",", "2"}], "}"}]}], "]"}], "//", 
        "Flatten"}]}]}], "\[IndentingNewLine]", "]"}]}]}]}]], "Input",
 InitializationCell->True,
 CellLabel->"In[13]:=",ExpressionUUID->"08257d49-4dcb-46f6-9efe-0fd2a79fa56e"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{"enable", " ", "graphics", " ", "mode"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"setgraphics", "[", 
     RowBox[{"1", ",", "0", ",", "8"}], "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "set", " ", "pen", " ", "to", " ", "white", " ", "and", " ", "background",
      " ", "to", " ", "black"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{"poke", "[", 
    RowBox[{"16^^6000", ",", 
     RowBox[{"Table", "[", 
      RowBox[{"16^^10", ",", "1000"}], "]"}]}], "]"}], "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"clear", " ", "the", " ", "screen"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{"poke", "[", 
    RowBox[{"16^^4000", ",", 
     RowBox[{"Table", "[", 
      RowBox[{"0", ",", "8000"}], "]"}]}], "]"}], "\n", 
   RowBox[{"(*", " ", 
    RowBox[{
     RowBox[{
      RowBox[{"and", " ", "run", " ", "Windows"}], "..."}], " ", "well", " ", 
     RowBox[{"almost", "!"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"open", " ", "icu64"}], " ", ">", " ", "graphics", " ", ">", 
        " ", "bitmaps"}], ",", " ", 
       RowBox[{"range", "=", 
        RowBox[{"4000", "..."}]}]}], ")"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Dynamic", "[", 
    RowBox[{
     RowBox[{"poke", "[", 
      RowBox[{"16^^4000", ",", 
       RowBox[{"bitmap", "[", 
        RowBox[{"CurrentScreenImage", "[", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"UpdateInterval", "\[Rule]", "2"}]}], "]"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"NOTE", ":", " ", 
      RowBox[{
      "Since", " ", "the", " ", "VICE", " ", "version", " ", "of", " ", 
       "icu64", " ", 
       RowBox[{"can", "'"}], "t", " ", "write", " ", "to", " ", "MMIO", " ", 
       "except", " ", "the", " ", "COLORRAM"}]}], ",", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"open", " ", "the", " ", "Graphics"}], " ", ">", " ", 
      "Bitmaps"}], ",", " ", 
     RowBox[{"Range", "=", 
      RowBox[{
       RowBox[{"4000", ".."}], " ", "to", " ", "see", " ", "the", " ", 
       "results", " ", "of", " ", "this", " ", 
       RowBox[{"cell", "."}]}]}]}], "\[IndentingNewLine]", 
    "*)"}]}]}]], "Input",ExpressionUUID->"ebd6a7ad-911e-4e87-b7c3-\
4b91e8239140"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "inject", " ", "some", " ", "6502", " ", "code", " ", "and", " ", "run", 
    " ", "it"}], " ", "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"addr", "=", "16^^c000"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"code", "=", "\"\<a2 00 fe 00 04 e8 4c 02 c0\>\""}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"poke", "[", 
    RowBox[{"addr", ",", 
     RowBox[{
      RowBox[{
       RowBox[{"FromDigits", "[", 
        RowBox[{"#", ",", "16"}], "]"}], "&"}], "/@", 
      RowBox[{"StringSplit", "[", "code", "]"}]}]}], "]"}], 
   "\[IndentingNewLine]", 
   RowBox[{"poke", "[", 
    RowBox[{"16^^314", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"addr", "~", "BitAnd", "~", "255"}], ",", 
       RowBox[{"addr", "~", "BitShiftRight", "~", "8"}]}], "}"}]}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[36]:=",ExpressionUUID->"5b0fea33-504b-48e7-b7a2-1610fd81cebb"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{824, 952},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
TrackCellChangeTimes->False,
SpellingDictionaries->{"CorrectWords"->{
 "Frodo", "Redpill", "Mathfigure", "exe", "ini"}},
FrontEndVersion->"12.0 for Microsoft Windows (64-bit) (April 8, 2019)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 115, 0, 67, "Section",ExpressionUUID->"1bcb10e7-e31c-4f8b-b039-b6bad58ce3df"],
Cell[698, 24, 725, 16, 204, "Text",ExpressionUUID->"64afa7cd-0d64-4e2d-8025-c684ef0be20d"],
Cell[CellGroupData[{
Cell[1448, 44, 91, 0, 53, "Subsection",ExpressionUUID->"ebe8a79b-4883-49ce-8e7e-73a14c3ed379"],
Cell[1542, 46, 914, 24, 197, "Input",ExpressionUUID->"f84757bd-866c-4276-ac84-d2d381af7710",
 InitializationCell->True]
}, Open  ]],
Cell[CellGroupData[{
Cell[2493, 75, 91, 0, 53, "Subsection",ExpressionUUID->"f85da292-8f9e-4f72-b74c-f62ad49ec021"],
Cell[2587, 77, 299, 7, 48, "Input",ExpressionUUID->"3752278d-d24b-4f09-ab7e-68d60110bd49"],
Cell[2889, 86, 324, 8, 48, "Input",ExpressionUUID->"635f98b5-0924-43e3-838a-883b72c49bd6"],
Cell[3216, 96, 427, 11, 48, "Input",ExpressionUUID->"a20a3cf2-503f-4cf6-8396-e8e5f5de7a58"],
Cell[3646, 109, 337, 8, 48, "Input",ExpressionUUID->"b9b24fb6-4014-400b-88cd-8547ae6257b7"]
}, Open  ]],
Cell[CellGroupData[{
Cell[4020, 122, 90, 0, 53, "Subsection",ExpressionUUID->"06f42f47-a6b1-40d1-881b-2cc1fb856233"],
Cell[4113, 124, 845, 24, 140, "Input",ExpressionUUID->"767f6b63-6b8e-4c2f-acaf-548ff3bd2886",
 InitializationCell->True],
Cell[4961, 150, 444, 12, 48, "Input",ExpressionUUID->"7f79e117-34bb-4de3-b4b2-e4c1dcf6b0a1"],
Cell[5408, 164, 503, 13, 67, "Input",ExpressionUUID->"39721d41-892f-4183-b86b-300a965719f2"],
Cell[5914, 179, 624, 18, 67, "Input",ExpressionUUID->"f680b9e3-40f2-44f3-b7ea-cf97a9bf28fd"],
Cell[6541, 199, 431, 12, 48, "Input",ExpressionUUID->"1e6e0221-998f-4e68-aae7-41b8d15806bd"],
Cell[6975, 213, 1127, 32, 67, "Input",ExpressionUUID->"8e620a02-1013-4ac5-930a-91079d55a0f7"],
Cell[8105, 247, 1347, 35, 143, "Input",ExpressionUUID->"5c23c635-f650-4e09-920f-e5a9bd1bdd1d"],
Cell[9455, 284, 4625, 131, 425, "Input",ExpressionUUID->"08257d49-4dcb-46f6-9efe-0fd2a79fa56e",
 InitializationCell->True],
Cell[14083, 417, 2404, 62, 257, "Input",ExpressionUUID->"ebd6a7ad-911e-4e87-b7c3-4b91e8239140"],
Cell[16490, 481, 970, 27, 105, "Input",ExpressionUUID->"5b0fea33-504b-48e7-b7a2-1610fd81cebb"]
}, Open  ]]
}, Open  ]]
}
]
*)

